#!/bin/sh

echo '
<html>
<body>
<h1> Allocators in libualloc </h1>
' > allocators.html

awk '

BEGINFILE { search=1 }
/[\/][*]/ { comment = 1; }

# Pick up allocator name and short desc
search && comment && /Allocator:/ {
	name=$0
	short=$0
	sub("^[ \t]*Allocator:*:[ \t]*", "", name)
	sub("[ \t].*", "", name);
	NAMES[name]++

	sub("^[ \t]*Allocator:*:[ \t]*", "", short)
	sub("^" name "[ \t:-]*", "", short)
	SHORT[name] = short
	next
}

search && comment && /:  / {
	field=$0
	value=$0
	sub("^[ \t]*", "", field)
	sub(":.*", "", field)
	sub("^[^:]*:[ \t]*", "", value)
	FIELDS[field]++
	DATA[name, field] = value
	next
}

/[*][\/]/ {
	comment = 0;
	search = 0
	name = ""
}

search && comment {
	DESC[name] = DESC[name] $0 "\n"
}

END {

	title="<h2>"
	ntitle="</h2>"
	alc="<h3>"
	nalc="</h3>"

	# generate comparison table

	print title "Comparison of allocators" ntitle
	print "<p><table border=1 cellspacing=0>"
	print "<tr><th> &nbsp;"
	for(f in FIELDS)
		print "<th>" f

	for(name in NAMES) {
		print "<tr><th>" name
		for(f in FIELDS)
			print "<td>" DATA[name, f]
	}
	print "</table>"

	print title "List of allocators" ntitle
	for(name in NAMES) {
		print alc name nalc
		print "<p> Method: " SHORT[name]
		print "<p>" DESC[name]
		print "<p><table border=1 cellspacing=0>"
		for(f in FIELDS)
			print "<tr><th>" f "<td>" DATA[name, f]
		print "</table>"
	}
}

' ../libualloc/*_api.h >> allocators.html

echo '
	</body>
	</html>
' >> allocators.html
