/*
 libualloc stacks example - this example code is placed in the public domain
 by Tibor 'Igor2' Palinkas in 2020

 This example is the same as a_stacks.c, for more explanation on what it
 does and why it is worth using stacks for this task, see the comments there.

 The purpose of this example is demonstration of low level mixed
 static/dynamic memory allocation: the first block is served from
 the stack, using pool[]. This should handle common execution just fine.
 Rarely, when a large amount of data is processed, pool[] runs out,
 further pages are allocated using malloc/free.
*/

#include <stdio.h>
#include <libualloc/libualloc.h>
#include <libualloc/stacks_api.h>

int main(int argc, char *argv[])
{
	char pool[4096];
	uall_sysalloc_t sys = {0};
	uall_stacks_t st = {0};
	double tmp, *d;

	/* initialize the low level allocator: pool[], then malloc()/free() */
	sys.alloc     = uall_static_stdlib_alloc;
	sys.free      = uall_static_stdlib_free;
	sys.user_data = pool;
	sys.page_size = sizeof(pool);

	/* initialize the stack allocator */
	st.sys = &sys;
	st.elem_size = sizeof(double);

	/* load input */
	while(scanf("%lf", &tmp) == 1) {
		d = uall_stacks_alloc(&st);
		if (d == NULL)
			return 1; /* in case sys.alloc a.k.a. malloc() fails on page allocation */
		*d = tmp;
	}

	/* print result */
	for(;;) {
		d = uall_stacks_top(&st);
		if (d == NULL)
			break;
		printf("%f\n", *d);
		uall_stacks_free(&st);
	}

	return 0;
}
