/*
 libualloc stacks example - this example code is placed in the public domain
 by Tibor 'Igor2' Palinkas in 2020

 This example demonstrates how to use the stacks allocator of libualloc
 static linking libualloc.a. The program doesn't do anything useful other
 than initializing a stack, the purpose of the code is to show that
 the compilation worked.

 Compilation command line; there are three options:

 1. libualloc is installed on the system

   cc -o l_lib l_lib.c -lualloc

 2. libualloc is installed as an user, e.g. in /home/foo/usr

   cc -o l_lib -I/home/foo/usr/include l_lib.c -L/home/foo/usr/lib -lualloc

 3. no installation; libualloc is embedded in the host program's source tree,
    under ../../libualloc

   cd ../../libualloc && make all
   cc -o l_lib -I../.. l_lib.c -L../../libualloc -lualloc

*/

#include <stdio.h>
#include <libualloc/libualloc.h>
#include <libualloc/stacks_api.h>

int main(int argc, char *argv[])
{
	uall_sysalloc_t sys = {0};
	uall_stacks_t st = {0};

	sys.alloc     = uall_stdlib_alloc;
	sys.free      = uall_stdlib_free;
	sys.page_size = 4096; /* how much memory to allocate to grow the stack */

	st.sys = &sys;
	st.elem_size = sizeof(double);

	if (uall_stacks_alloc(&st) != 0) {
		printf("success!\n");
		return 0;
	}

	printf("runtime error\n");
	return 1;
}
